      subroutine blocking_method
c------------------------------------------------------
c     INPUT:
c     iuwt        -> unit file
c     npoints     -> number of consecutive measurements
c                    of some fluctuating quantity
c     x(npoints)  -> consecutive measurements of some 
c                    fluctuating quantity
c     OUTPUT:
c     nexp        -> number of blocking transformations
c     npont(nexp) -> data set in each block
c     aver        -> fluctuating quantity average value
c     sg(3)-> standard deviation and its error
c-------------------------------------------------------- 

      use datos, only : npoints, x, nexp, aver, npont, sg, faterr, 
     &                  iuwt, warning
      implicit none
      logical :: endlog
      integer :: nval, nj, i, j, ind1, ind2, ier
      real*8 :: var, xold(npoints), xnew(npoints), denom, aver2
      real*8, parameter :: tol = 1.0d-6

      write(*,*)
      write(*,'(3x,a)') 'STEP2: BLOCKING ALGORITHM'
      write(*,'(3x,a)') '========================='
c
c-----Obtention of nexp 
c
      nexp = 0
      endlog = .true.
      nval = npoints
      do while (endlog)
        endlog = mod(nval,2).eq.0 .and. nval.gt.2
        if (endlog) then
          nexp = nexp + 1
          nval = nval/2
        endif
      enddo
      write(*,*)
      write(*,'(3x,a,1x,i5)') 'Number of times the algorithm is'
     &                      //' repeated ', nexp
c
c-----Fluctuating quantity average value
c
      aver = 0.0d0
      do i = 1, npoints
        aver = aver + x(i)
      enddo
      aver = aver/npoints
      write(*,*)
      write(*,'(3x,a,1x,e18.10)') 'Fluctuating quantity average' 
     &                           //' value ',aver
c
c-----Allocate arrays
c
      allocate (npont(nexp), stat=ier)
      if (ier.ne.0) then
        call error ('blocking_method', 'Cannot allocate npont', 
     &              faterr)
      endif
      allocate (sg(3), stat=ier)
      if (ier.ne.0) then
        call error ('blocking_method', 'Cannot allocate sg', faterr)
      endif
c
c-----Obtention of npont
c
      write(*,*)
      write(*,'(3x,a)') 'Number of discrete values in each'
     &                      //' step'
      write(*,'(3x,a)') 'Step---number of values'
      write(*,'(3x,a)') '-----------------------' 
      write(*,'(3x,i5,1x,i10)') 0,npoints
      nval = npoints
      do i = 1, nexp
        nval = nval/2
        npont(i) = nval
        write(*,'(3x,i5,1x,i10)') i,npont(i)
      enddo
c
c-----Initial estimate standard deviation
c
      write(*,*)
      write(*,'(3x,a)') 'Algorithm: Step--n--c0/(n-1)---standard'
     &                //' deviation'
      write(*,'(3x,a)') '--------------------------------------'
     &                //'----------'
      write(iuwt,'(1x,a,1x,e18.10)') '# average value :',aver
      write(iuwt,'(1x,a)') '# number of block---standard deviation'
     &                   //' (minimum, value, maximum)'
      var = 0.0d0
      do i = 1, npoints
        var = var + (x(i)-aver)*(x(i)-aver)
      enddo
      var = var/(npoints*(npoints-1.0d0))
      sg(1) = sqrt(var)
      write(*,'(3x,i10,1x,i10,1x,e18.10,1x,e18.10)') 0,npoints,var,
     &                                               sg(1)
      denom = sqrt(2.0d0*(npoints-1.0d0))
      var = sg(1)/denom
      sg(2) = sg(1) + var
      sg(3) = sg(1) - var 
      write(iuwt,'(1x,i5,3(1x,e18.10))') 0,sg(3),sg(1),sg(2)     
c
c-----Blocking Method
c
      do i = 1, npoints
        xold(i) = x(i)
      enddo 
      do i = 1, nexp
c.......We transform the data set into half as large data set
        do j = 1, npont(i)
          ind1 = j+j-1
          ind2 = j+j
          xnew(j) = (xold(ind1)+xold(ind2))/2.0d0
        enddo
c.......Fluctuating quantity average value
        aver2 = 0.0d0
        do j = 1, npont(i)
          aver2 = aver2 + xnew(j)
        enddo
        aver2 = aver2/npont(i)
        if (abs(aver-aver2).ge.tol*aver) then
          call error ('blocking_method', 'warning average value',
     &                 warning)      
           write (*,'(3x,a,1x,e18.10)') 'aver = ',aver
           write (*,'(3x,a,1x,e18.10)') 'aver2 = ',aver2
        endif   
c.......Estimate for standard deviation
        var = 0.0d0
        do j = 1, npont(i)
          var = var + (xnew(j)-aver)*(xnew(j)-aver)
        enddo
        var = var/(npont(i)*(npont(i)-1.0d0))
        sg(1) = sqrt(var)
        write(*,'(3x,i10,1x,i10,1x,e18.10,1x,e18.10)') i,npont(i),var,
     &                                             sg(1)
        denom = sqrt(2.0d0*(npont(i)-1.0d0))
        var = sg(1)/denom
        sg(2) = sg(1) + var
        sg(3) = sg(1) - var
        write(iuwt,'(1x,i5,3(1x,e18.10))') i,sg(3),sg(1),sg(2)
        do j = 1, npont(i)
          xold(j) = xnew(j)
        enddo 
      enddo  
c
c-----End
c
      return
      end
