      subroutine s_routine
c------------------------------------------------------
c     INPUT:
c     iuwt        -> unit file
c     npoints     -> number of consecutive measurements
c                    of some fluctuating quantity
c     x(npoints)  -> consecutive measurements of some 
c                    fluctuating quantity
c     OUTPUT:
c     nexp        -> number of blocking transformations
c     aver        -> fluctuating quantity average value
c     sg(3)-> standard deviation and its error
c-------------------------------------------------------- 

      use datos, only : npoints, x, nexp, aver, sg, faterr, 
     &                  iuwt
      implicit none
      logical :: endlog
      integer :: nval, nb, taub, ier, i, j, k, l
      real*8 :: var, var2, aver2

      write(*,*)
      write(*,'(3x,a)') 'STEP2: STATISTICAL INEFFICIENCY'
      write(*,'(3x,a)') '==============================='
c
c-----Obtention of nexp 
c
      nexp = 1
      endlog = .true.
      nval = npoints
      do while (endlog)
        endlog = mod(nval,2).eq.0 .and. nval.gt.2
        if (endlog) then
          nexp = nexp + 1
          nval = nval/2
        endif
      enddo
      write(*,*)
      write(*,'(3x,a,1x,i5)') 'Number of times the algorithm is'
     &                      //' repeated ', nexp
c
c-----Fluctuating quantity average value
c
      aver = 0.0d0
      do i = 1, npoints
        aver = aver + x(i)
      enddo
      aver = aver/npoints
      write(*,*)
      write(*,'(3x,a,1x,e18.10)') 'Fluctuating quantity average' 
     &                          //' value ',aver
c
c-----Variance
c
      var = 0.0d0
      do i = 1, npoints
        var = var + (x(i)-aver)*(x(i)-aver)
      enddo  
      var = var/npoints   
      write(*,'(3x,a,1x,e18.10)') 'Variance ',var
c
c-----Allocate arrays
c
      allocate (sg(1), stat=ier)
      if (ier.ne.0) then
        call error ('blocking_method', 'Cannot allocate sg', faterr)
      endif
c
c-----Statistical inefficiency
      write(*,*)
      write(iuwt,'(1x,a,1x,i10)') '# number of consecutive' 
     &                             //' measurements :',npoints
      write(iuwt,'(1x,a,1x,e18.10)') '# average value :',aver
      write(iuwt,'(1x,a,1x,e18.10)') '# variance :',var
      write(iuwt,'(1x,a)') '# sqrt(block length)---statistical'
     &                   //' inefficiency'
      do i = nexp,1,-1        
c.......Number of blocks and elements in each block
        nb = 2**i
        taub = 2**(nexp-i) 
        write(*,'(1x,a,1x,i10)') 'number of blocks ',nb
        write(*,'(1x,a,1x,i10)') 'number of elements in each block ',
     &                            taub
c.......Mean value, variance and statistical inefficiency
        var2 = 0.0d0
        k = 0
        do j = 1, nb
          aver2 = 0.0d0
          do l = 1, taub
            k = k + 1
            aver2 = aver2 + x(k)
          enddo
          aver2 = aver2/taub
          var2 = var2 + (aver2-aver)*(aver2-aver)
        enddo
        var2 = var2/nb
        sg(1) = taub*var2/var
        write(*,'(1x,a,1x,e18.10)') 'statistical inefficiency ',sg(1)
        write(iuwt,'(1x,e18.10,1x,e18.10)') dsqrt(dble(taub)),sg(1)
      enddo  
c
c-----End
c
      return
      end
